/* $Header: /CVSROOT/debris/drisexec.h,v 1.3 2005/07/25 03:22:19 tino Exp $
 *
 * DebRIS: DEBian Remote Install Scripter
 * invokation of other processes
 *
 * $Log: drisexec.h,v $
 * Revision 1.3  2005/07/25 03:22:19  tino
 * extended
 *
 * Revision 1.2  2005/07/24 22:17:46  tino
 * using snprintf instead of sprintf.
 * printf replaced by info
 *
 * Revision 1.1  2005/06/01 02:12:34  tino
 * first version.  As SirbeD is missing, this just does nothing yet.
 */

#define	FORK_MAXARGS	100

static void
printer(const char *buf, size_t len)
{
  int	pos;

  for (pos=0; pos<len; )
    {
      int	i;

      for (i=pos; i<len && (isprint(buf[i]) || isspace(buf[i])); i++);
      if (i==pos)
	info("[%02x]", (unsigned char)buf[pos++]);
      else
	{
	  info("%.*s", (int)(i-pos), buf+pos);
	  pos	= i;
	}
    }
}

static void
dris_vout(const char *s, va_list list)
{
  size_t	old;

  old	= config.out->len;
  dris_buf_printf(config.out, s, list);
  printer(config.out->buf+old, config.out->len-old);
}

static void
dris_out(const char *s, ...)
{
  va_list	list;

  va_start(list, s);
  dris_vout(s, list);
  va_end(list);
}

static void
dris_vout_error(const char *s, va_list list)
{
  vfprintf(stderr, s, list);
}

static void
dris_out_error(const char *s, ...)
{
  va_list	list;

  va_start(list, s);
  dris_vout_error(s, list);
  va_end(list);
}

static pid_t
forkcmd(void)
{
  pid_t	chld;
  char	tmp[100];
  int	fd;

  snprintf(tmp, sizeof tmp, "/tmp/debris.%lu.out.tmp", config.pid);
  if ((fd=open(tmp, O_CREAT|O_TRUNC|O_RDWR, 0600))<0)
    ex(tmp);
  if (fd<0)
    ex(tmp);
  info2("[%lu] exec: %s\n", config.pid, config.in->buf);
  if ((chld=fork())==0)
    {
      char	*args[FORK_MAXARGS];
      int	i;
      char	*cmd;

      /* assign stdout	*/
      if (fd!=1)
	dup2(fd, 1);
      if (fd>1)
	close(fd);

      /* assign stdin	*/
      snprintf(tmp, sizeof tmp, "/tmp/debris.%lu.in.tmp", config.pid);
      if ((fd=open(tmp, O_CREAT|O_TRUNC|O_RDWR, 0600))<0)
	ex(tmp);
      if (config.in->len > config.data)
	writeall(fd, config.in->buf+config.data, config.in->len-config.data);
      if (lseek(fd, 0, SEEK_SET))
	ex("lseek");
      if (fd)
	dup2(fd, 0);
      if (fd>1)
	close(fd);

      /* create arg list	*/
      cmd	= config.in->buf;
      for (i=0; (args[i++]=nextarg(&cmd))!=0 && cmd && i<FORK_MAXARGS-1; );
      args[i]	= 0;

      /* run	*/
      execv(args[0], args+1);
      ex(args[0]);
    }

  if (chld==(pid_t)-1)
    {
      perror(config.in->buf);
      close(fd);
      return 0;
    }
  config.fd	= fd;
  return chld;
}

static int
waitfor(pid_t chld)
{
  if (!chld || chld==(pid_t)-1)
    return -1;
  if (!config.verbose && config.fd>=0)
    {
      close(config.fd);
      config.fd	= -1;
    }
  for (;;)
    {
      pid_t	pid;
      int	retval;

      alarm(1);
      pid	= waitpid((pid_t)0, &retval, 0);
      alarm(0);
      if (pid==chld)
	{
	  if (WIFEXITED(retval))
	    return WEXITSTATUS(retval);
	  if (WIFSIGNALED(retval))
	    {
	      info("-> signal %d\n", WTERMSIG(retval));
	      return -2;
	    }
	  else if (WIFSTOPPED(retval))
	    info("-> jobcontrol %d\n", WSTOPSIG(retval));
	  else
	    {
	      info("-> unknown reason %d\n", retval);
	      return -3;
	    }
	}
      if (config.fd>=0)
	{
	  char	buf[BUFSIZ];
	  int	got;

	  if ((got=read(config.fd, buf, sizeof buf))==0)
	    continue;

	  if (got>0)
	    printer(buf, got);
	  else if (errno!=EINTR && errno!=EAGAIN)
	    {
	      close(config.fd);
	      config.fd	= -1;
	    }
	}
    }
}
