/* $Header: /CVSROOT/debris/dristool.h,v 1.3 2005/07/25 03:15:43 tino Exp $
 *
 * DebRIS: DEBian Remote Install Scripter
 * Helping functions
 *
 * $Log: dristool.h,v $
 * Revision 1.3  2005/07/25 03:15:43  tino
 * quote processing added in nextarg
 *
 * Revision 1.2  2005/07/24 22:14:43  tino
 * added info2
 *
 * Revision 1.1  2005/06/01 02:12:35  tino
 * first version.  As SirbeD is missing, this just does nothing yet.
 */

#define DP(X)	do { dprintf X; } while (0)

static void
info2(const char *s, ...)
{
  va_list	list;

  va_start(list, s);
  vfprintf(stderr, s, list);
  va_end(list);
}

static void
info(const char *s, ...)
{
  va_list	list;

  va_start(list, s);
  vprintf(s, list);
  va_end(list);
}

static void
verbose(const char *s, ...)
{
  va_list	list;

  if (!config.verbose)
    return;

  va_start(list, s);
  vprintf(s, list);
  va_end(list);
  fflush(stdout);
}

static void
dprintf(const char *s, ...)
{
  va_list	list;

  if (!config.debug)
    return;

  fprintf(stderr, "[");
  va_start(list, s);
  vfprintf(stderr, s, list);
  va_end(list);
  fprintf(stderr, "]\n");
}


static void
ex(const char *s)
{
  int	e;

  e	= errno;
  perror(s);
  printf("error: %s: %s\n", s, strerror(e));
  exit(-1);
}

static char *
nextarg(char **ptr)
{
  char	*ret, *cmd, *out, quote;

  for (cmd= *ptr;; cmd++)
    {
      if (!*cmd)
	return 0;
      if (!isspace(*cmd))
	break;
    }
  ret	= cmd;

  quote	= 0;
  out	= ret;
  for (;;)
    {
      char	c;

      switch (c= *cmd++)
	{
	case 0:
	  *ptr	= 0;
	  *out	= 0;
	  return ret;

	case '"':
	case '\'':
	  if (!quote)
	    {
	      quote	= c;
	      continue;
	    }
	  if (c==quote)
	    {
	      quote	= 0;
	      continue;
	    }
	  break;

	case ' ':
	case '\t':
	case '\v':
	case '\r':
	case '\n':
	  if (!quote)
	    {
	      *ptr	= cmd+1;
	      *out	= 0;
	      return ret;
	    }
	}
      *out++	= c;
    }
}

static void
writeall(int fd, const void *buf, size_t len)
{
  while (len)
    {
      int	put;

      if ((put=write(fd, buf, len))>0)
	{
	  len			-= put;
	  ((const char *)buf)	+= put;
	  continue;
	}
      if (!put)
	ex("writeall: unexpected EOF, filesystem full?");
      if (errno!=EINTR && errno!=EAGAIN)
	ex("writeall");
    }
}
